package com.zhiwei_tech.openapidemo;

import android.content.Context;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.RotateAnimation;
import android.widget.AbsListView;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * Created by zhenggary on 15/9/2.
 */
public class RefreshListView extends ListView implements AbsListView.OnScrollListener {
    View refresh_head;//顶部布局文件
    int headerHight;//顶部布局文件高度
    int firstVisibleItem;//当前界面可见的第一个ITEM的位置
    boolean isRemark;//标记,当前是在listView最顶端向下拉
    int startY;//向下拉时的Y值
    int scrollState;//listView当前滚动状态
    IRefreshListener iRefreshListener;//刷新数据接口
    int state;//标记当前状态
    final int NONE = 0;//正常状态
    final int PULL = 1;//提示下拉状态
    final int RELESE = 2;//提示松开状态
    final int REFRESHING = 3;//正在刷新状态

    public RefreshListView(Context context) {
        super(context);
        initView(context);
    }

    public RefreshListView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initView(context);
    }

    public RefreshListView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initView(context);
    }

    public RefreshListView(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        initView(context);
    }


    @Override
    public void onScrollStateChanged(AbsListView view, int scrollState) {
        this.scrollState = scrollState;
    }

    @Override
    public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount,
                         int totalItemCount) {
        this.firstVisibleItem = firstVisibleItem;

    }

    @Override
    public boolean onTouchEvent(MotionEvent ev) {
        switch (ev.getAction()) {
            case MotionEvent.ACTION_DOWN:
                if (firstVisibleItem == 0) {
                    isRemark = true;
                    startY = (int) ev.getY();
                }
                break;
            case MotionEvent.ACTION_MOVE:
                onMove(ev);
                break;
            case MotionEvent.ACTION_UP:
                if (state == RELESE) {
                    state = REFRESHING;
                    refreshViewByState();
                    //加载最新数据
                    iRefreshListener.onRefresh();
                } else if (state == PULL) {
                    state = NONE;
                    isRemark = false;
                    refreshViewByState();
                }
                break;
        }
        return super.onTouchEvent(ev);
    }

    public interface IRefreshListener {
        public void onRefresh();
    }

    public void setInterface(IRefreshListener iRefreshListener) {
        this.iRefreshListener = iRefreshListener;
    }

    /**
     * 刷新完成方法
     */
    public void refreshComplete() {
        state = NONE;
        isRemark = false;
        refreshViewByState();
        TextView lastUpdateTime = (TextView) refresh_head.findViewById(R.id.refresh_time);
        SimpleDateFormat format = new SimpleDateFormat("yyyy年mm月dd日 hh:mm:ss");
        Date date = new Date(System.currentTimeMillis());
        String time = format.format(date);
        lastUpdateTime.setText("更新时间：" + time);
    }

    /**
     * 设置下拉刷新顶部上边距
     *
     * @param topPadding
     */
    private void topPadding(int topPadding) {
        refresh_head.setPadding(refresh_head.getPaddingLeft(), topPadding,
                refresh_head.getPaddingRight(), refresh_head.getPaddingBottom());
        refresh_head.invalidate();
    }

    /**
     * 初始化布局文件
     *
     * @param context
     */
    private void initView(Context context) {
        LayoutInflater inflater = LayoutInflater.from(context);
        refresh_head = inflater.inflate(R.layout.head, null);
        measureView(refresh_head);
        headerHight = refresh_head.getMeasuredHeight();
        topPadding(-headerHight);
        this.addHeaderView(refresh_head);
        this.setOnScrollListener(this);
    }

    /**
     * 通知父布局，占用的宽，高
     *
     * @param view
     */
    private void measureView(View view) {
        ViewGroup.LayoutParams p = view.getLayoutParams();
        if (p == null) {
            p = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT);
        }
        int width = ViewGroup.getChildMeasureSpec(0, 0, p.width);
        int height;
        int tempHeight = p.height;
        if (tempHeight > 0) {
            height = MeasureSpec.makeMeasureSpec(tempHeight, MeasureSpec.EXACTLY);
        } else {
            height = MeasureSpec.makeMeasureSpec(0, MeasureSpec.UNSPECIFIED);
        }
        view.measure(width, height);
    }

    /**
     * 判断移动过程中的操作
     *
     * @param ev
     */
    private void onMove(MotionEvent ev) {
        if (!isRemark) {
            return;
        }
        int tempY = (int) ev.getY();//当前Y值
        int space = tempY - startY;//移动的距离
        int topPadding = space - headerHight;
        switch (state) {
            case NONE:
                if (space > 0) {
                    state = PULL;
                    refreshViewByState();
                }
                break;

            case PULL:
                topPadding(topPadding);
                if (space > headerHight + 30
                        && scrollState == SCROLL_STATE_TOUCH_SCROLL) {
                    state = RELESE;
                    refreshViewByState();
                }
                break;
            case RELESE:
                topPadding(topPadding);
                if (space < headerHight + 30) {
                    state = PULL;
                    refreshViewByState();
                } else if (space <= 0) {
                    state = NONE;
                    isRemark = false;
                    refreshViewByState();
                }
                break;


        }
    }

    /**
     * 根据当前状态改变界面显示
     */
    private void refreshViewByState() {
        TextView tip = (TextView) refresh_head.findViewById(R.id.tip);
        ImageView arrow = (ImageView) refresh_head.findViewById(R.id.arrow);
        ProgressBar progress = (ProgressBar) refresh_head.findViewById(R.id.progress);
        RotateAnimation animation_back = new RotateAnimation(0, 180,
                RotateAnimation.RELATIVE_TO_SELF, 0.5f, RotateAnimation.RELATIVE_TO_SELF, 0.5f);
        animation_back.setDuration(200);
        animation_back.setFillAfter(true);
        RotateAnimation animation_go = new RotateAnimation(180, 0,
                RotateAnimation.RELATIVE_TO_SELF, 0.5f, RotateAnimation.RELATIVE_TO_SELF, 0.5f);
        animation_go.setDuration(200);
        animation_go.setFillAfter(true);
        switch (state) {
            case NONE:
                arrow.clearAnimation();
                topPadding(-headerHight);
                break;
            case PULL:
                arrow.setVisibility(View.VISIBLE);
                progress.setVisibility(View.GONE);
                tip.setText("下拉可以刷新");
                arrow.clearAnimation();
                arrow.setAnimation(animation_go);
                break;
            case RELESE:
                arrow.setVisibility(View.VISIBLE);
                progress.setVisibility(View.GONE);
                tip.setText("松开可以刷新");
                arrow.clearAnimation();
                arrow.setAnimation(animation_back);
                break;
            case REFRESHING:
                topPadding(50);
                arrow.setVisibility(View.GONE);
                progress.setVisibility(View.VISIBLE);
                tip.setText("正在刷新");
                arrow.clearAnimation();
                break;
        }
    }

}
